proc NastranBase2PostCmdAssignMat { GDN w Menu geomlist } {
    $Menu del 0 end
    if { [catch { set whatuse [lindex [.gid.central.s info Project] 4] }] } {
	set whatuse GEOMETRYUSE
    }
    switch $whatuse {
	GEOMETRYUSE {
	    foreach i $geomlist {
		$Menu add command -label $i -command "DWAssignMaterials $GDN $w $i"
	    }
	}
	MESHUSE {
	    $Menu add command -label Elements -command "DWAssignMaterials $GDN $w Elements"
	}
    }
}
proc NastranBasePostCmdAssignMat { GDN w Menu } {
    upvar \#0 $GDN GidData

    set MatName $GidData(MAT,matname)
    set _MatName [DWSpace2Under $MatName]
    set property [lindex [.central.s info Materials $_MatName] 4]
    set book [.central.s info Materials $_MatName BOOK]

    set LinesMats [list BAR BEAM TUBE VISCOUS_DAMPER SPRING DOF_SPRING]
    set SurfacesMats [list MEMBRANE PLATE PLANE_STRAIN SHEAR_PANEL BENDING_ONLY LAMINATE]
    set VolumesMats [list TETRAHEDRON HEXAHEDRON] 
    if { [lsearch $LinesMats $property] != -1 } {
	NastranBase2PostCmdAssignMat $GDN $w $Menu Lines
    } elseif { [lsearch $SurfacesMats $property] != -1 } {
	NastranBase2PostCmdAssignMat $GDN $w $Menu Surfaces
    } elseif { [lsearch $VolumesMats $property] != -1 } {
	NastranBase2PostCmdAssignMat $GDN $w $Menu Volumes
    } elseif { $book != "Material" && $book != "Tables"} {
	NastranPostCmdAssignMat $GDN $w $Menu
    } else {
	$Menu del 0 end
	$Menu add command -label "This material or table cannot be assigned"
    }
}

proc NastranBaseDWDrawMaterials { GDN w what } {
    upvar \#0 $GDN GidData

    set MatName $GidData(MAT,matname)
    set _MatName [DWSpace2Under $MatName]
    set book [.central.s info Materials $_MatName BOOK]
    if { $book != "Material" && $book != "Tables"} {
	NastranDWDrawMaterials $GDN $w $what
    } else {
	WarnWin "This material or table cannot be drawn"
    }
}

proc NastranBaseDWUnassignMat { w GDN } {
   upvar \#0 $GDN GidData

    set MatName $GidData(MAT,matname)
    set _MatName [DWSpace2Under $MatName]
    
    set book [.central.s info Materials $_MatName BOOK]
    if { $book != "Material" && $book != "Tables"} {
	NastranDWUnassignMat $w $GDN
    } else {
	WarnWin "This material or table cannot be unassigned"
    }
}

proc InitGIDProject { dir } {
    global MenuNames MenuEntries MenuCommands MenuAcceler
    global MenuNamesP MenuEntriesP MenuCommandsP MenuAccelerP
    global ProgramName VersionNumber ProblemTypePriv NastranPriv
    global GidPriv
    
	
    set ProgramName "GiD-NASTRAN Interface" 
    set VersionNumber 2.0

    catch { unset ProblemTypePriv }

    set ProblemTypePriv(problemtypedir) $dir
    catch {
		GidShowBook intvdata "" 0
    }
    set ipos [lsearch $MenuNames Help]
    if { $ipos != -1 } {
	set MenuEntries($ipos) [linsert $MenuEntries($ipos) 0 "GiD-NASTRAN Interface tutorials" \
		"GiD-NASTRAN Interface help"]
	set MenuCommands($ipos) [linsert $MenuCommands($ipos) 0 \
		[list -np- HelpWindow "CUSTOM_HELP_FILE" \
		[file join $dir ../info nastran_tutorials.html]] \
		[list -np- HelpWindow "CUSTOM_HELP_REFERENCE" \
		[file join $dir ../info nastran_toc.html]]]
	set MenuAcceler($ipos) [linsert $MenuAcceler($ipos) 0 "" ""]

	lappend MenuEntries($ipos) "About GiD-NASTRAN Interface"
	lappend MenuCommands($ipos) [list -np- Splash $dir 0]
	lappend MenuAcceler($ipos) ""
    }
    set ipos [lsearch $MenuNamesP Help]
    if { $ipos != -1 } {
	set MenuEntriesP($ipos) [linsert $MenuEntriesP($ipos) 0 "GiD-NASTRAN Interface tutorials" \
		"GiD-NASTRAN Interface help"]
	set MenuCommandsP($ipos) [linsert $MenuCommandsP($ipos) 0 \
		[list -np- HelpWindow "CUSTOM_HELP_FILE" \
		[file join $dir ../info nastran_tutorials.html]] \
		[list -np- HelpWindow "CUSTOM_HELP_REFERENCE" \
		[file join $dir ../info nastran_toc.html]]]
	set MenuAccelerP($ipos) [linsert $MenuAccelerP($ipos) 0 "" ""]

	lappend MenuEntriesP($ipos) "About GiD-NASTRAN Interface"
	lappend MenuCommandsP($ipos) [list -np- Splash $dir 0]
	lappend MenuAccelerP($ipos) ""
    }
	
    set ipos [lsearch $MenuNamesP Files]
    set iposin [lsearch $MenuEntriesP($ipos) Import]
    set MenuEntriesP($ipos,$iposin) [linsert $MenuEntriesP($ipos,$iposin) 2 "FEMAP Info..."]
    set MenuCommandsP($ipos,$iposin) [linsert $MenuCommandsP($ipos,$iposin) 2 [list -np- FemapInfo ]]
    set MenuEntriesP($ipos,$iposin) [linsert $MenuEntriesP($ipos,$iposin) 3 "Import PUNCH"]
    set MenuCommandsP($ipos,$iposin) [linsert $MenuCommandsP($ipos,$iposin) 3 [list -np- PCH2GiD $dir 1 0]]
    set MenuEntriesP($ipos,$iposin) [linsert $MenuEntriesP($ipos,$iposin) 4 "Import PUNCH(Smooth)"]
    set MenuCommandsP($ipos,$iposin) [linsert $MenuCommandsP($ipos,$iposin) 4 [list -np- PCH2GiD $dir 1 1]]
    #set MenuAccelerP($ipos,$iposin) [linsert $MenuAccelerP($ipos,$iposin) 2 ""]
    
    set GidVersion [.gid.central.s info GiDVersion]
    set GidVersion [string trim $GidVersion]
    set GidVersion [string index $GidVersion 0]
    
    if { $GidVersion <= 6 } {
	set iposin2 [lsearch $MenuEntriesP($ipos,$iposin) "Image"]
	set MenuEntriesP($ipos,$iposin,$iposin2) $MenuEntriesP($ipos,$iposin,[expr $iposin2-2])
	catch {unset MenuEntriesP($ipos,$iposin,[expr $iposin2-2])}
	set MenuCommandsP($ipos,$iposin,$iposin2) $MenuCommandsP($ipos,$iposin,[expr $iposin2-2])
	catch {unset MenuCommandsP($ipos,$iposin,[expr $iposin2-2])}
    }
    set ipos [lsearch $MenuNames Files]
    if { $ipos != -1 } {
	set ipos2 [lsearch $MenuEntries($ipos) *Export*]
	if { $ipos2 != -1 } {
	    set ipos3 [lsearch $MenuEntries($ipos,$ipos2) "*Calculation file*"]
	    if { $ipos3 != -1 } {
		set MenuCommands($ipos,$ipos2) [lreplace $MenuCommands($ipos,$ipos2) \
		        $ipos3 $ipos3 "-np- WriteCalcFileCheck"]
	    }
	}
    }
    
    set ipos [lsearch $MenuNames Calculate]
    if { $ipos != -1 } {
	set ipos2 [lsearch $MenuEntries($ipos) Calculate]
	if { $ipos2 != -1 } {
	    set MenuCommands($ipos) [lreplace $MenuCommands($ipos) \
		        $ipos2 $ipos2 "-np- WriteCalcFileCheck"]
	}
    }
    foreach i [list PostCmdAssignMat DWDrawMaterials DWUnassignMat] {
	if { [info command Nastran$i] == "" } {
	    rename $i Nastran$i
	    rename NastranBase$i $i
	}
    }

    if { ![info exists  NastranPriv(infosplash)] } {
	catch { Splash $dir }
	set NastranPriv(infosplash) 1
    }

    foreach i [list helpviewer.tcl nastranpch2gid.tcl function.tcl supergrid.tcl \
	    verifymodel.tcl tkcheckdata.tcl nassections.tcl composite.tcl \
	    verify_properties.tcl BasWriter.tcl nasmat.tcl base64.tcl] {
	source [file join $dir  scripts $i]
    }
   
    MyBitmaps $dir

    GidChangeDataLabel "Problem Data" ""
    GidChangeDataLabel "Conditions" ""
    GidChangeDataLabel "Materials" ""
    GidChangeDataLabel "Interval" ""
    
    GidAddUserDataOptionsMenu "Boundary Conditions" "NastranBoundaryconditionsSubmenu %W" 2
    GidAddUserDataOptionsMenu "Loads" "NastranLoadsSubmenu %W" 3
    GidAddUserDataOptions "Advanced Conditions" "GidOpenConditions Advanced_Conditions" 4
    GidAddUserDataOptions "Materials" "GidOpenMaterials Material" 5
    GidAddUserDataOptionsMenu "Properties" "NastranPropertiesSubmenu %W" 6
    GidAddUserDataOptionsMenu "Analysis Design" "NastranAnalysisSubmenu %W" 7
    GidAddUserDataOptionsMenu "Problem data" "NastranProblemdataSubmenu %W" 8
    GidAddUserDataOptions "Verify Properties" VerifyProperty 9
    GidAddUserDataOptions "---" "" 10  
	
    set GidPriv(ProgName) $ProgramName
    ChangeWindowTitle
    CreateTopMenus
}
proc NastranPropertiesSubmenu { w } {
    $w del 0 end
    $w add command -label "Property" -command [list GidOpenMaterials Property ]
    $w add command -label "Local Axes" -command [list GidOpenConditions Local_Axes ]
    $w configure -postcommand ""
}
proc NastranLoadsSubmenu { w } {
    $w del 0 end
    $w add command -label "Static Loads" -command [list GidOpenConditions Static_Loads ]
    $w add command -label "Dynamics Loads" -command [list GidOpenConditions Dynamics_Loads ]
    $w add command -label "Tables" -command [list GidOpenMaterials Tables ]
    $w configure -postcommand ""
}
proc NastranBoundaryconditionsSubmenu { w } {
    $w del 0 end
    $w add command -label "Constraints" -command [list GidOpenConditions Constraints ]
    $w add command -label "Connections" -command [list GidOpenConditions Connections ]
    $w configure -postcommand ""
}
proc NastranProblemdataSubmenu { w } {
    $w del 0 end
    $w add command -label "Executive Control" -command [list GidOpenProblemData Executive_Control]
    $w add command -label "Case Control" -command [list GidOpenProblemData Case_Control]
    $w add command -label "Parameters" -command [list GidOpenProblemData Parameters]
    $w configure -postcommand ""
}
proc NastranAnalysisSubmenu { w } {
    $w del 0 end
    $w add command -label "Dynamics" -command [list GidOpenProblemData Dynamics]
    $w add command -label "Buckling" -command [list GidOpenProblemData Buckling]
    $w add command -label "NonLinear" -command [list GidOpenProblemData NonLinear]
    $w configure -postcommand ""
}


proc EndGIDProject {} {
    global ProblemTypePriv
    
    if { [info exists ProblemTypePriv(condfuns)] } {
	foreach i $ProblemTypePriv(condfuns) {
	    rename $i {}
	    rename $i-base $i
	}
    }
    catch { destroy .gid.loadcases }
    foreach i [list PostCmdAssignMat DWDrawMaterials DWUnassignMat] {
	if { [info command Nastran$i] != "" } {
	    rename $i ""
	    rename Nastran$i $i
	}
    }
    global GidPriv
    set GidPriv(ProgName) GiD
    ChangeWindowTitle ""
    EndMyBitmaps
}

proc Splash { dir { autodestroy 1 } } {
    global GIDDEFAULT VersionNumber ProgramName

    if { [.gid.central.s disable windows] } { return }

    if { [ winfo exist .splash]} {
	destroy .splash
	update
    }

    toplevel .splash
    wm withdraw .splash
	
    set im [image create photo -file [ file join $dir images \
	    [string tolower $ProgramName]-open.gif]]
    set x [expr [winfo screenwidth .splash]/2-[image width $im]/2]
    set y [expr [winfo screenheight .splash]/2-[image height $im]/2]

    wm geom .splash +$x+$y
    wm transient .splash .gid
    wm overrideredirect .splash 1
    pack [label .splash.l -image $im -relief ridge -bd 2]
    
    label .splash.lv -text $VersionNumber -foreground white -relief raised -bd 3 \
	    -background blue4 -font "verdana 8"
    place .splash.lv -x 62 -y 62
    bind .splash <1> "destroy .splash"
    bind .splash <KeyPress> "destroy .splash"
    grab .splash
    focus .splash
    update
    wm deiconify .splash
    raise .splash .gid
    update
    if { $autodestroy } {
	after 4000 "if { [ winfo exist .splash] } { 
	    destroy .splash
	}"
    }
    
}

proc PCH2GiD { dir reenterpost  smooth } {

    global DefaultSearchDirectory

    if { ![info exists DefaultSearchDirectory] || \
	     ![file isdirectory $DefaultSearchDirectory]} {
	set DefaultSearchDirectory [pwd]
    }
    set deft $DefaultSearchDirectory

    set filein [tk_getOpenFile -defaultextension ".pch"  -initialdir $deft \
		    -parent .gid -filetypes {{{PUNCH files} {.pch} } {{All files} {*} }}]
    if { $filein == "" } {
	return
    }
    set DefaultSearchDirectory [file dirname $filein]

    set aa [.central.s info Project]
    set ProjectName [lindex $aa 1]
    if { [file extension $ProjectName] == ".gid" } {
	set ProjectName [file root $ProjectName]
    }
    if { $ProjectName == "UNNAMED" } {
	tk_dialogRAM .gid.tmpwin error \
		  "Before importing, a project title is needed. Save project to get it" \
		  error 0 OK
	  return
    }

    set directory $ProjectName.gid
    if { [file pathtype $directory] == "relative" } {
	set directory [file join [pwd] $directory]
    }
    set fileout [file join $directory [file root [file tail $ProjectName]].flavia.res]
    
    PCH2GiD::TranslateW  $filein $fileout $smooth
    
    if { $reenterpost } {
	set DisableGraphics [.central.s disable graphics]
	set DisableWarnLine [.central.s disable warnline]
	.central.s disable graphics 1
	.central.s disable warnline 1
	.central.s process escape escape escape escape preprocess yes 
	.central.s process escape escape escape escape postprocess yes
	if { !$DisableGraphics } { .central.s disable graphics 0 }
	if { !$DisableWarnLine } { .central.s disable warnline 0 }
    }
}

proc FemapInfo { } {

    tk_dialogRAMFull .gid.alele {information window} \
	"It is only possible to import FEMAP\nneutral files in ASCII format.\nTo\
	obtain a FEMAP ASCII file in NE/NASTRAN go to NASTRAN Editor:\nMenu\
	Setup, Default Analysis Options.\nSelect RSLTFILETYPE,FEMAP ASCII\
	in Output Control Directives." \
	"" "" gidquestionhead 0 OK 
}

proc MyBitmaps { dir { type "DEFAULT INSIDELEFT"} } {
    global MyBitmapsNames MyBitmapsCommands MyBitmapsHelp ProblemTypePriv ProgramName

    set MyBitmapsNames(0) "images/constraints.gif images/connections.gif \
	images/loads.gif images/dynamicloads.gif images/table.gif images/materials.gif \
	images/section.gif images/localaxes.gif images/executivecontrol.gif images/casecontrol.gif images/dynamics.gif \
	images/buckling.gif images/nonlinear.gif images/drawloads.gif images/mesh.gif images/calculate.gif"
    set MyBitmapsCommands(0) [list [list -np- GidOpenConditions Constraints] \
	    [list -np- GidOpenConditions Connections] \
	    [list -np- GidOpenConditions Static_Loads] \
	    [list -np- GidOpenConditions Dynamics_Loads] \
	    [list -np- GidOpenMaterials Tables] \
	    [list -np- GidOpenMaterials Material] \
	    [list -np- GidOpenMaterials Property] \
	    [list -np- GidOpenConditions Local_Axes] \
	    [list -np- GidOpenProblemData Executive_Control] \
	    [list -np- GidOpenProblemData Case_Control] \
	    [list -np- GidOpenProblemData Dynamics] \
	    [list -np- GidOpenProblemData Buckling] \
	    [list -np- GidOpenProblemData  NonLinear] \
	    [list -np- .central.s process escape escape escape escape data conditions drawcond -DrawAll-] \
	    "Meshing generate" \
	    [list -np- WriteCalcFileCheck]]
    set MyBitmapsHelp(0) { "Assign Constraints" "Assign Inter-element Connections"  "Assign Static Loads" \
	    "Assign Dynamic Loads" "Create Interpolation table values\n to define dynamic loads" \
	    "Define Material Properties" "Assign Properties" \
	    "Define Local Axes for 1-D elements" \
	    "Define parameters of Executive Control section" \
	    "Define parameters of Case Control section" \
	    "Define parameters of Dynamic Analysis" \
	    "Define parameters of Buckling Analysis" \
	    "Define parameters of NonLinear Analysis" \
	    "Draw Loads and Constraints" "Generate Mesh" "Write Nastran Input file" }
    
#     set MyBitmapsNames(0,2) ""
#     set MyBitmapsCommands(0,2) ""

    # prefix values:
    #          Pre        Only active in the preprocessor
    #          Post       Only active in the postprocessor
    #          PrePost    Active Always

    set prefix Pre

    set ProblemTypePriv(toolbarwin) [CreateOtherBitmaps MyBar "My toolbar" \
	    MyBitmapsNames MyBitmapsCommands \
	    MyBitmapsHelp $dir "MyBitmaps [list $dir]" $type $prefix]
    AddNewToolbar "$ProgramName bar" ${prefix}MyBarWindowGeom \
	    "MyBitmaps [list $dir]"

    # button number 3
#     set ProblemTypePriv(loadcasewin) $ProblemTypePriv(toolbarwin).2
# 
#     if { [info exists ProblemTypePriv(loadcaseimg1)] } {
#         ChangeToLoadCase 1
#     }

}

proc EndMyBitmaps {} {
    global ProblemTypePriv ProgramName
    
    ReleaseToolbar "$ProgramName bar"
    rename MyBitmaps ""
    
    catch { destroy $ProblemTypePriv(toolbarwin) }
}
